#include <klocalizedstring.h>

/********************************************************************************
** Form generated from reading UI file 'jsconsole.ui'
**
** Created by: Qt User Interface Compiler version 5.9.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_JSCONSOLE_H
#define UI_JSCONSOLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "numberedtextview.h"

QT_BEGIN_NAMESPACE

class Ui_KJSConsole
{
public:
    QAction *actionOpenScript;
    QAction *actionCloseScript;
    QAction *actionQuit;
    QAction *actionRun;
    QAction *actionRunTo;
    QAction *actionStep;
    QAction *actionStop;
    QWidget *centralWidget;
    QVBoxLayout *vboxLayout;
    QSplitter *splitter;
    QSplitter *splitter1;
    QTreeView *mObjectModel;
    NumberedTextView *mInput;
    QTextBrowser *mConsole;
    QHBoxLayout *hboxLayout;
    QComboBox *mCommand;
    QPushButton *mExecute;
    QMenuBar *menuBar;
    QMenu *menuFile;
    QToolBar *mainToolBar;
    QStatusBar *statusBar;

    void setupUi(QMainWindow *KJSConsole)
    {
        if (KJSConsole->objectName().isEmpty())
            KJSConsole->setObjectName(QStringLiteral("KJSConsole"));
        KJSConsole->resize(800, 600);
        actionOpenScript = new QAction(KJSConsole);
        actionOpenScript->setObjectName(QStringLiteral("actionOpenScript"));
        actionCloseScript = new QAction(KJSConsole);
        actionCloseScript->setObjectName(QStringLiteral("actionCloseScript"));
        actionQuit = new QAction(KJSConsole);
        actionQuit->setObjectName(QStringLiteral("actionQuit"));
        actionRun = new QAction(KJSConsole);
        actionRun->setObjectName(QStringLiteral("actionRun"));
        const QIcon icon = QIcon(QString::fromUtf8(":/images/start.png"));
        actionRun->setIcon(icon);
        actionRunTo = new QAction(KJSConsole);
        actionRunTo->setObjectName(QStringLiteral("actionRunTo"));
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/runto.png"));
        actionRunTo->setIcon(icon1);
        actionStep = new QAction(KJSConsole);
        actionStep->setObjectName(QStringLiteral("actionStep"));
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/step.png"));
        actionStep->setIcon(icon2);
        actionStop = new QAction(KJSConsole);
        actionStop->setObjectName(QStringLiteral("actionStop"));
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/stop.png"));
        actionStop->setIcon(icon3);
        centralWidget = new QWidget(KJSConsole);
        centralWidget->setObjectName(QStringLiteral("centralWidget"));
        vboxLayout = new QVBoxLayout(centralWidget);
#ifndef UI_Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MAC
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName(QStringLiteral("vboxLayout"));
        splitter = new QSplitter(centralWidget);
        splitter->setObjectName(QStringLiteral("splitter"));
        splitter->setOrientation(Qt::Vertical);
        splitter1 = new QSplitter(splitter);
        splitter1->setObjectName(QStringLiteral("splitter1"));
        QSizePolicy sizePolicy(static_cast<QSizePolicy::Policy>(7), static_cast<QSizePolicy::Policy>(5));
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(splitter1->sizePolicy().hasHeightForWidth());
        splitter1->setSizePolicy(sizePolicy);
        splitter1->setOrientation(Qt::Horizontal);
        mObjectModel = new QTreeView(splitter1);
        mObjectModel->setObjectName(QStringLiteral("mObjectModel"));
        mObjectModel->setAlternatingRowColors(true);
        mObjectModel->setSelectionMode(QAbstractItemView::NoSelection);
        splitter1->addWidget(mObjectModel);
        mInput = new NumberedTextView(splitter1);
        mInput->setObjectName(QStringLiteral("mInput"));
        QSizePolicy sizePolicy1(static_cast<QSizePolicy::Policy>(5), static_cast<QSizePolicy::Policy>(5));
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(mInput->sizePolicy().hasHeightForWidth());
        mInput->setSizePolicy(sizePolicy1);
        splitter1->addWidget(mInput);
        splitter->addWidget(splitter1);
        mConsole = new QTextBrowser(splitter);
        mConsole->setObjectName(QStringLiteral("mConsole"));
        QSizePolicy sizePolicy2(static_cast<QSizePolicy::Policy>(7), static_cast<QSizePolicy::Policy>(7));
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(mConsole->sizePolicy().hasHeightForWidth());
        mConsole->setSizePolicy(sizePolicy2);
        mConsole->setFrameShape(QFrame::StyledPanel);
        mConsole->setFrameShadow(QFrame::Sunken);
        splitter->addWidget(mConsole);

        vboxLayout->addWidget(splitter);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName(QStringLiteral("hboxLayout"));
        mCommand = new QComboBox(centralWidget);
        mCommand->setObjectName(QStringLiteral("mCommand"));
        QSizePolicy sizePolicy3(static_cast<QSizePolicy::Policy>(1), static_cast<QSizePolicy::Policy>(0));
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(mCommand->sizePolicy().hasHeightForWidth());
        mCommand->setSizePolicy(sizePolicy3);
        mCommand->setEditable(true);
        mCommand->setDuplicatesEnabled(true);

        hboxLayout->addWidget(mCommand);

        mExecute = new QPushButton(centralWidget);
        mExecute->setObjectName(QStringLiteral("mExecute"));

        hboxLayout->addWidget(mExecute);


        vboxLayout->addLayout(hboxLayout);

        KJSConsole->setCentralWidget(centralWidget);
        menuBar = new QMenuBar(KJSConsole);
        menuBar->setObjectName(QStringLiteral("menuBar"));
        menuBar->setGeometry(QRect(0, 0, 800, 26));
        menuFile = new QMenu(menuBar);
        menuFile->setObjectName(QStringLiteral("menuFile"));
        KJSConsole->setMenuBar(menuBar);
        mainToolBar = new QToolBar(KJSConsole);
        mainToolBar->setObjectName(QStringLiteral("mainToolBar"));
        mainToolBar->setMovable(false);
        mainToolBar->setOrientation(Qt::Horizontal);
        mainToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
        KJSConsole->addToolBar(static_cast<Qt::ToolBarArea>(4), mainToolBar);
        statusBar = new QStatusBar(KJSConsole);
        statusBar->setObjectName(QStringLiteral("statusBar"));
        statusBar->setGeometry(QRect(0, 581, 800, 19));
        KJSConsole->setStatusBar(statusBar);

        menuBar->addAction(menuFile->menuAction());
        menuFile->addAction(actionOpenScript);
        menuFile->addAction(actionCloseScript);
        menuFile->addSeparator();
        menuFile->addAction(actionQuit);
        mainToolBar->addAction(actionRun);
        mainToolBar->addAction(actionRunTo);
        mainToolBar->addAction(actionStep);
        mainToolBar->addAction(actionStop);

        retranslateUi(KJSConsole);

        QMetaObject::connectSlotsByName(KJSConsole);
    } // setupUi

    void retranslateUi(QMainWindow *KJSConsole)
    {
        KJSConsole->setWindowTitle(tr2i18n("MainWindow", Q_NULLPTR));
        actionOpenScript->setText(tr2i18n("Open Script", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionOpenScript->setStatusTip(tr2i18n("Open a script...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
#ifndef UI_QT_NO_SHORTCUT
        actionOpenScript->setShortcut(tr2i18n("Ctrl+O", Q_NULLPTR));
#endif // QT_NO_SHORTCUT
        actionCloseScript->setText(tr2i18n("Close Script", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionCloseScript->setStatusTip(tr2i18n("Close script...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
        actionQuit->setText(tr2i18n("Quit", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionQuit->setStatusTip(tr2i18n("Quit application...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
        actionRun->setText(tr2i18n("Run", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionRun->setStatusTip(tr2i18n("Run script...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
        actionRunTo->setText(tr2i18n("Run To...", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionRunTo->setStatusTip(tr2i18n("Run to breakpoint...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
        actionStep->setText(tr2i18n("Step", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionStep->setStatusTip(tr2i18n("Step to next line...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
        actionStop->setText(tr2i18n("Stop", Q_NULLPTR));
#ifndef UI_QT_NO_STATUSTIP
        actionStop->setStatusTip(tr2i18n("Step execution...", Q_NULLPTR));
#endif // QT_NO_STATUSTIP
        mExecute->setText(tr2i18n("Execute", Q_NULLPTR));
        menuFile->setTitle(tr2i18n("File", Q_NULLPTR));
    } // retranslateUi

};

namespace Ui {
    class KJSConsole: public Ui_KJSConsole {};
} // namespace Ui

QT_END_NAMESPACE

#endif // JSCONSOLE_H

