/****************************************************************************
** Meta object code from reading C++ file 'numberedtextview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kjsembed-5.39.0/examples/kjsconsole/numberedtextview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'numberedtextview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_NumberBar_t {
    QByteArrayData data[1];
    char stringdata0[10];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NumberBar_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NumberBar_t qt_meta_stringdata_NumberBar = {
    {
QT_MOC_LITERAL(0, 0, 9) // "NumberBar"

    },
    "NumberBar"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NumberBar[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void NumberBar::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObject NumberBar::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_NumberBar.data,
      qt_meta_data_NumberBar,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *NumberBar::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NumberBar::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NumberBar.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int NumberBar::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_NumberedTextView_t {
    QByteArrayData data[12];
    char stringdata0[98];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NumberedTextView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NumberedTextView_t qt_meta_stringdata_NumberedTextView = {
    {
QT_MOC_LITERAL(0, 0, 16), // "NumberedTextView"
QT_MOC_LITERAL(1, 17, 10), // "mouseHover"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 4), // "word"
QT_MOC_LITERAL(4, 34, 3), // "pos"
QT_MOC_LITERAL(5, 38, 11), // "textChanged"
QT_MOC_LITERAL(6, 50, 5), // "added"
QT_MOC_LITERAL(7, 56, 7), // "removed"
QT_MOC_LITERAL(8, 64, 4), // "text"
QT_MOC_LITERAL(9, 69, 11), // "currentLine"
QT_MOC_LITERAL(10, 81, 8), // "stopLine"
QT_MOC_LITERAL(11, 90, 7) // "bugLine"

    },
    "NumberedTextView\0mouseHover\0\0word\0pos\0"
    "textChanged\0added\0removed\0text\0"
    "currentLine\0stopLine\0bugLine"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NumberedTextView[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       4,   44, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       1,    2,   32,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    3,   37,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QPoint, QMetaType::QString,    4,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,    4,    6,    7,

 // properties: name, type, flags
       8, QMetaType::QString, 0x00095103,
       9, QMetaType::Int, 0x00095103,
      10, QMetaType::Int, 0x00095103,
      11, QMetaType::Int, 0x00095103,

       0        // eod
};

void NumberedTextView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        NumberedTextView *_t = static_cast<NumberedTextView *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->mouseHover((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->mouseHover((*reinterpret_cast< const QPoint(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->textChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (NumberedTextView::*_t)(const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&NumberedTextView::mouseHover)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (NumberedTextView::*_t)(const QPoint & , const QString & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&NumberedTextView::mouseHover)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        NumberedTextView *_t = static_cast<NumberedTextView *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->text(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->currentLine(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->stopLine(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->bugLine(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        NumberedTextView *_t = static_cast<NumberedTextView *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setText(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setCurrentLine(*reinterpret_cast< int*>(_v)); break;
        case 2: _t->setStopLine(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setBugLine(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

const QMetaObject NumberedTextView::staticMetaObject = {
    { &QFrame::staticMetaObject, qt_meta_stringdata_NumberedTextView.data,
      qt_meta_data_NumberedTextView,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *NumberedTextView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NumberedTextView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NumberedTextView.stringdata0))
        return static_cast<void*>(this);
    return QFrame::qt_metacast(_clname);
}

int NumberedTextView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
   else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void NumberedTextView::mouseHover(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void NumberedTextView::mouseHover(const QPoint & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
